@@ -20,10 +20,17 @@ Public License along with Dominate.  If not, see
 
 import copy
 import numbers
-from collections import defaultdict, namedtuple, Callable
+from collections import defaultdict, namedtuple
 from functools import wraps
 import threading
 
+try:
+  # Python 3
+  from collections.abc import Callable
+except ImportError:
+  # Python 2.7
+  from collections import Callable
+
 try:
   basestring = basestring
 except NameError: # py3
@@ -36,6 +43,7 @@ try:
 except ImportError:
   greenlet = None
 
+
 def _get_thread_context():
   context = [threading.current_thread()]
   if greenlet:
@@ -50,11 +58,11 @@ class dom_tag(object):
                      # modified
   is_inline = False
 
-  frame = namedtuple('frame', ['tag', 'items', 'used'])
 
   def __new__(_cls, *args, **kwargs):
     '''
-    Check if bare tag is being used a a decorator.
+    Check if bare tag is being used a a decorator
+    (called with a single function arg).
     decorate the function and return
     '''
     if len(args) == 1 and isinstance(args[0], Callable) \
@@ -68,9 +76,10 @@ class dom_tag(object):
       return f
     return object.__new__(_cls)
 
+
   def __init__(self, *args, **kwargs):
     '''
-    Creates a new tag. Child tags should be passed as aruguments and attributes
+    Creates a new tag. Child tags should be passed as arguments and attributes
     should be passed as keyword arguments.
 
     There is a non-rendering attribute which controls how the tag renders:
@@ -82,7 +91,6 @@ class dom_tag(object):
     self.attributes = {}
     self.children   = []
     self.parent     = None
-    self.document   = None
 
     # Does not insert newlines on all children if True (recursive attribute)
     self.is_inline = kwargs.pop('__inline', self.is_inline)
@@ -93,33 +101,40 @@ class dom_tag(object):
       self.add(*args)
 
     for attr, value in kwargs.items():
-      self.set_attribute(*dom_tag.clean_pair(attr, value))
+      self.set_attribute(*type(self).clean_pair(attr, value))
 
     self._ctx = None
     self._add_to_ctx()
 
-  def _add_to_ctx(self):
-    ctx = dom_tag._with_contexts[_get_thread_context()]
-    if ctx and ctx[-1]:
-      self._ctx = ctx[-1]
-      ctx[-1].items.append(self)
 
-  # stack of (root_tag, [new_tags], set(used_tags))
+  # context manager
+  frame = namedtuple('frame', ['tag', 'items', 'used'])
+  # stack of frames
   _with_contexts = defaultdict(list)
 
+  def _add_to_ctx(self):
+    stack = dom_tag._with_contexts.get(_get_thread_context())
+    if stack:
+      self._ctx = stack[-1]
+      stack[-1].items.append(self)
+
+
   def __enter__(self):
-    ctx = dom_tag._with_contexts[_get_thread_context()]
-    ctx.append(dom_tag.frame(self, [], set()))
+    stack = dom_tag._with_contexts[_get_thread_context()]
+    stack.append(dom_tag.frame(self, [], set()))
     return self
 
+
   def __exit__(self, type, value, traceback):
-    ctx = dom_tag._with_contexts[_get_thread_context()]
-    slf, items, used = ctx[-1]
-    ctx[-1] = None
-    for item in items:
-      if item in used: continue
+    thread_id = _get_thread_context()
+    stack = dom_tag._with_contexts[thread_id]
+    frame = stack.pop()
+    for item in frame.items:
+      if item in frame.used: continue
       self.add(item)
-    ctx.pop()
+    if not stack:
+      del dom_tag._with_contexts[thread_id]
+
 
   def __call__(self, func):
     '''
@@ -139,6 +154,7 @@ class dom_tag(object):
         return func(*args, **kwargs) or tag
     return f
 
+
   def set_attribute(self, key, value):
     '''
     Add or update the value of an attribute.
@@ -152,6 +168,7 @@ class dom_tag(object):
           'child tags and attributes, respectively.')
   __setitem__ = set_attribute
 
+
   def delete_attribute(self, key):
     if isinstance(key, int):
       del self.children[key:key+1]
@@ -159,17 +176,6 @@ class dom_tag(object):
       del self.attributes[key]
   __delitem__ = delete_attribute
 
-  def setdocument(self, doc):
-    '''
-    Creates a reference to the parent document to allow for partial-tree
-    validation.
-    '''
-    # assume that a document is correct in the subtree
-    if self.document != doc:
-      self.document = doc
-      for i in self.children:
-        if not isinstance(i, dom_tag): return
-        i.setdocument(doc)
 
   def add(self, *args):
     '''
@@ -181,16 +187,15 @@ class dom_tag(object):
         obj = str(obj)
 
       if isinstance(obj, basestring):
-        obj = escape(obj)
+        obj = util.escape(obj)
         self.children.append(obj)
 
       elif isinstance(obj, dom_tag):
-        ctx = dom_tag._with_contexts[_get_thread_context()]
-        if ctx and ctx[-1]:
-          ctx[-1].used.add(obj)
+        stack = dom_tag._with_contexts.get(_get_thread_context())
+        if stack:
+          stack[-1].used.add(obj)
         self.children.append(obj)
         obj.parent = self
-        obj.setdocument(self.document)
 
       elif isinstance(obj, dict):
         for attr, value in obj.items():
@@ -208,18 +213,22 @@ class dom_tag(object):
 
     return args
 
+
   def add_raw_string(self, s):
     self.children.append(s)
 
+
   def remove(self, obj):
     self.children.remove(obj)
 
+
   def clear(self):
     for i in self.children:
       if isinstance(i, dom_tag) and i.parent is self:
         i.parent = None
     self.children = []
 
+
   def get(self, tag=None, **kwargs):
     '''
     Recursively searches children for tags of a certain
@@ -246,6 +255,7 @@ class dom_tag(object):
         results.extend(child.get(tag, **kwargs))
     return results
 
+
   def __getitem__(self, key):
     '''
     Returns the stored value of the specified attribute or child
@@ -268,12 +278,14 @@ class dom_tag(object):
           'child tags and attributes, respectively.')
   __getattr__ = __getitem__
 
+
   def __len__(self):
     '''
     Number of child elements.
     '''
     return len(self.children)
 
+
   def __bool__(self):
     '''
     Hack for "if x" and __len__
@@ -281,12 +293,14 @@ class dom_tag(object):
     return True
   __nonzero__ = __bool__
 
+
   def __iter__(self):
     '''
     Iterates over child elements.
     '''
     return self.children.__iter__()
 
+
   def __contains__(self, item):
     '''
     Checks recursively if item is in children tree.
@@ -294,6 +308,7 @@ class dom_tag(object):
     '''
     return bool(self.get(item))
 
+
   def __iadd__(self, obj):
     '''
     Reflexive binary addition simply adds tag as a child.
@@ -306,15 +321,16 @@ class dom_tag(object):
     return self.render()
   __str__ = __unicode__
 
+
   def render(self, indent='  ', pretty=True, xhtml=False):
     data = self._render([], 0, indent, pretty, xhtml)
     return u''.join(data)
 
+
   def _render(self, sb, indent_level, indent_str, pretty, xhtml):
     pretty = pretty and self.is_pretty
 
-    t = type(self)
-    name = getattr(t, 'tagname', t.__name__)
+    name = getattr(self, 'tagname', type(self).__name__)
 
     # Workaround for python keywords and standard classes/methods
     # (del, object, input)
@@ -326,21 +342,25 @@ class dom_tag(object):
     sb.append(name)
 
     for attribute, value in sorted(self.attributes.items()):
-      sb.append(' %s="%s"' % (attribute, escape(unicode(value), True)))
+      if value in (False, None):
+        continue
+      val = unicode(value) if isinstance(value, util.text) and not value.escape else util.escape(unicode(value), True)
+      sb.append(' %s="%s"' % (attribute, val))
 
     sb.append(' />' if self.is_single and xhtml else '>')
 
-    if not self.is_single:
-      inline = self._render_children(sb, indent_level + 1, indent_str, pretty, xhtml)
+    if self.is_single:
+      return sb
 
-      if pretty and not inline:
-        sb.append('\n')
-        sb.append(indent_str * indent_level)
+    inline = self._render_children(sb, indent_level + 1, indent_str, pretty, xhtml)
+    if pretty and not inline:
+      sb.append('\n')
+      sb.append(indent_str * indent_level)
 
-      # close tag
-      sb.append('</')
-      sb.append(name)
-      sb.append('>')
+    # close tag
+    sb.append('</')
+    sb.append(name)
+    sb.append('>')
 
     return sb
 
@@ -358,6 +378,7 @@ class dom_tag(object):
 
     return inline
 
+
   def __repr__(self):
     name = '%s.%s' % (self.__module__, type(self).__name__)
 
@@ -371,30 +392,41 @@ class dom_tag(object):
 
     return '<%s at %x: %s, %s>' % (name, id(self), attributes, children)
 
+
   @staticmethod
   def clean_attribute(attribute):
     '''
-    Since some attributes match python keywords we prepend them with
-    underscores. Python also does not support colons in keywords so underscores
-    mid-attribute are replaced with colons.
+    Normalize attribute names for shorthand and work arounds for limitations
+    in Python's syntax
     '''
-    # shorthand
+
+    # Shorthand
     attribute = {
       'cls': 'class',
       'className': 'class',
       'class_name': 'class',
+      'klass': 'class',
       'fr': 'for',
       'html_for': 'for',
       'htmlFor': 'for',
+      'phor': 'for',
     }.get(attribute, attribute)
 
-    # Workaround for python's reserved words
-    if attribute[0] == '_': attribute = attribute[1:]
+    # Workaround for Python's reserved words
+    if attribute[0] == '_':
+      attribute = attribute[1:]
+
+    # Workaround for dash
+    special_prefix = any([attribute.startswith(x) for x in ('data_', 'aria_')])
+    if attribute in set(['http_equiv']) or special_prefix:
+      attribute = attribute.replace('_', '-').lower()
+
+    # Workaround for colon
+    if attribute.split('_')[0] in ('xlink', 'xml', 'xmlns'):
+      attribute = attribute.replace('_', ':', 1).lower()
+
+    return attribute
 
-    # Workaround for inability to use colon in python keywords
-    if attribute in set(['http_equiv']) or attribute.startswith('data_'):
-      return attribute.replace('_', '-').lower()
-    return attribute.replace('_', ':').lower()
 
   @classmethod
   def clean_pair(cls, attribute, value):
@@ -411,25 +443,35 @@ class dom_tag(object):
     if value is True:
       value = attribute
 
-    if value is False:
-      value = "false"
+    # Ignore `if value is False`: this is filtered out in render()
 
     return (attribute, value)
 
 
+_get_current_none = object()
+def get_current(default=_get_current_none):
+  '''
+  get the current tag being used as a with context or decorated function.
+  if no context is active, raises ValueError, or returns the default, if provided
+  '''
+  h = _get_thread_context()
+  ctx = dom_tag._with_contexts.get(h, None)
+  if ctx:
+    return ctx[-1].tag
+  if default is _get_current_none:
+    raise ValueError('no current context')
+  return default
+
+
 def attr(*args, **kwargs):
   '''
   Set attributes on the current active tag context
   '''
-  ctx = dom_tag._with_contexts[_get_thread_context()]
-  if ctx and ctx[-1]:
-    dicts = args + (kwargs,)
-    for d in dicts:
-      for attr, value in d.items():
-        ctx[-1].tag.set_attribute(*dom_tag.clean_pair(attr, value))
-  else:
-    raise ValueError('not in a tag context')
+  c = get_current()
+  dicts = args + (kwargs,)
+  for d in dicts:
+    for attr, value in d.items():
+      c.set_attribute(*dom_tag.clean_pair(attr, value))
 
 
-# escape() is used in render
-from .util import escape
+from . import util
