@@ -1,3 +1,5 @@
+import pytest
+
 from dominate.tags import *
 
 def test_dom():
@@ -5,12 +7,21 @@ def test_dom():
   with container.add(div(id='base')) as dom:
     s1 = span('Hello', id='span1')
     s2 = span('World', id='span2')
-  
+
   s3 = span('foobar', id='span3')
   dom.appendChild(s3)
-
-  assert container.getElementById('base') is dom  
+  assert container.getElementById('base') is dom
   assert container.getElementById('span1') is s1
   assert container.getElementById('span3') is s3
   assert container.getElementsByTagName('span') == [s1, s2, s3]
   assert container.getElementsByTagName('SPAN') == [s1, s2, s3]
+
+
+def test_element():
+  d = div(
+    span(id='a'),
+    span(id='a'),
+  )
+  with pytest.raises(ValueError):
+    d.getElementById('a')
+
