@@ -1,6 +1,9 @@
+import pytest
+
 from dominate.tags import *
 from dominate import util
 
+
 def test_include():
   import os
   try:
@@ -18,6 +21,7 @@ def test_include():
     except:
       pass
 
+
 def test_system():
   d = div()
   d += util.system('echo Hello World')
@@ -27,6 +31,26 @@ def test_system():
 def test_unescape():
   assert util.unescape('&amp;&lt;&gt;&#32;') == '&<> '
 
+
 def test_url():
   assert util.url_escape('hi there?') == 'hi%20there%3F'
-  assert util.url_unescape('hi%20there%3f') == 'hi there?'
\ No newline at end of file
+  assert util.url_unescape('hi%20there%3f') == 'hi there?'
+
+
+def test_container():
+  d = div()
+  with d:
+    with util.container():
+      pass
+  assert d.render() == '<div></div>'
+
+
+  d = div()
+  with d:
+    with util.container():
+      h1('a')
+  assert d.render() == \
+'''<div>
+  <h1>a</h1>
+</div>'''
+
